SystemOrganization addCategory: #MGPlotting!

Morph subclass: #MGPlotMorph
	instanceVariableNames: 'relations plotRectangle coordinatesUnitRectangle backgroundColor axesColor xLabelsAbove yLabelsLeft xLabelsMaxDecimalPlaces yLabelsMaxDecimalPlaces plotForm plottingProcess extentSharedQueue plotFormSharedQueue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGPlotMorph commentStamp: 'mg 12/11/2013 00:12' prior: 0!
A MGPlotMorph is a Morph used for creating plots. To open the morph do (you can also do it after adding the relations):
	mgPlotMorph := MGPlotMorph new.
	mgPlotMorph openInWorld.
Now you need to add some relations. Either you create the MGRelation object (of one of its nonabstract subclasses), or you can use some helper methods in the relations protocol. To give an example:
	mgRelation := MGRelationOnLeftToRightBlock new.
	mgRelation block: [:x|x squared].
	mgPlotMorph addRelation: mgRelation.
For different subclasses of MGRelation see MGRelation comment.

Instance Variables
	relations:		<OrderedCollection of MGRelations>
	plotRectangle:	<Rectangle>			
	coordinatesUnitRectangle:	<Rectangle>						
	backgroundColor:	<Color>							
	axisColor:	<Color>
	xLabelsAbove:	<Boolean> 
	yLabelsLeft:	<Boolean>
	xLabelsMaxDecimalPlaces:		<Integer>
	yLabelsMaxDecimalPlaces:		<Integer>
	plotForm:	<Form>
	plottingProcess:	<Process>		
	extentSharedQueue:	<SharedQueue2 of Points>		
	plotFormSharedQueue:	<SharedQueue2 of Forms>

relations
	- an ordered collection of relations to plot

plotRectangle
	- a rectangle describing the plotting area

coordinatesUnitRectangle
	- a rectangle describing the units  of the axes and the origin of the plot

backgroundColor
	- the color of the background
	
axesColor
	- the color of the axes and the  numbers

xLabelsAbove
	- true if x labels should be above the axis, false if below
	
yLabelsLeft
	- true if y labels should be to the left of the axis, false if to the right
	
xLabelsMaxDecimalPlaces
	- maximal number of decimal places in x labels
	
yLabelsMaxDecimalPlaces
	- maximal number of decimal places in y labels

plotForm
	- a form used for caching the plot data

plottingProcess
	- the plotting process
	
extentSharedQueue
	- used for interprocess communication with the plotting process
		
plotFormSharedQueue
	- used for interprocess communication with the plotting process!

!MGPlotMorph class methodsFor: 'examples' stamp: 'mg 12/11/2013 02:21'!
example1
	|mgPlotMorph mgRelationOnLeftToRightBlock|
	"Do it!! MGPlotMorph example1."
	
	mgPlotMorph := MGPlotMorph new.
	mgPlotMorph openInWindowLabeled: 'A plot'.	

	mgRelationOnLeftToRightBlock := MGRelationOnLeftToRightBlock new.
	mgRelationOnLeftToRightBlock continuous.
	mgRelationOnLeftToRightBlock color: Color blue.
	mgRelationOnLeftToRightBlock block: [:x|x squared].
	mgPlotMorph addRelation: mgRelationOnLeftToRightBlock.		
	
	! !

!MGPlotMorph methodsFor: 'relations' stamp: 'mg 12/7/2013 13:48'!
addRelation: anMGRelation
	"Add an MGRelation to the plot."
	relations add: anMGRelation.
	anMGRelation addDependent: self.
	self rememberToReplot.
	^ anMGRelation! !

!MGPlotMorph methodsFor: 'relations' stamp: 'mg 12/10/2013 11:17'!
addRelationOn2DBlock: aBlock
	"Convenience method for fast adding an MGRelationOn2DBlock."
	^self addRelation: (MGRelationOn2DBlock new block: aBlock)! !

!MGPlotMorph methodsFor: 'relations' stamp: 'mg 12/10/2013 11:17'!
addRelationOnCollection: aCollection
	"Convenience method for fast adding an MGRelationOnCollection."
	^self addRelation: (MGRelationOnCollection new collection: aCollection).
! !

!MGPlotMorph methodsFor: 'relations' stamp: 'mg 12/10/2013 11:18'!
addRelationOnCollectionWithIcons: aCollection
	"Convenience method for fast adding an MGRelationOnCollectionWithIcons."
	^self addRelation: (MGRelationOnCollectionWithIcons new collection: aCollection).
! !

!MGPlotMorph methodsFor: 'relations' stamp: 'mg 12/10/2013 11:18'!
addRelationOnLeftToRightBlock: aBlock
	"Convenience method for fast adding an MGRelationOnLeftToRightBlock."
	^self addRelation:(MGRelationOnLeftToRightBlock new block: aBlock).! !

!MGPlotMorph methodsFor: 'relations' stamp: 'mg 12/10/2013 11:18'!
addRelationOnTopToBottomBlock: aBlock
	"Convenience method for fast adding an MGRelationOnTopToBottomBlock."
	^self addRelation: (MGRelationOnTopToBottomBlock new block: aBlock).
! !

!MGPlotMorph methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:23'!
axesColor
	"Return the color of the axes and the numbers"
	^ axesColor! !

!MGPlotMorph methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:23'!
axesColor: aColor
	"Set the color of the axes and the numbers"
	axesColor := aColor.
	self rememberToReplot.! !

!MGPlotMorph methodsFor: 'accessing' stamp: 'mg 12/7/2013 13:52'!
backgroundColor
	"Return the color of the background"
	^ backgroundColor! !

!MGPlotMorph methodsFor: 'accessing' stamp: 'mg 12/7/2013 13:52'!
backgroundColor: aColor
	"Set the color of the background"
	backgroundColor := aColor.
	self rememberToReplot.! !

!MGPlotMorph methodsFor: 'private form drawing' stamp: 'mg 12/10/2013 23:57'!
drawCoordinatesOn: aCanvas transformation: aPlotToFormTransformation
	|transformedCoordinatesUnitRectangle xAxisY yAxisX maxXPoint maxYPoint xUnit yUnit|
	transformedCoordinatesUnitRectangle := aPlotToFormTransformation 
		toFormRectangle: coordinatesUnitRectangle.
	xAxisY := transformedCoordinatesUnitRectangle origin y.
	yAxisX := transformedCoordinatesUnitRectangle origin x.
	maxXPoint := aCanvas extent x @ xAxisY.			
	maxYPoint := yAxisX @ aCanvas extent y.				
	xUnit := transformedCoordinatesUnitRectangle width.		
	yUnit := transformedCoordinatesUnitRectangle height negated.			
	self 
		drawXAxisOn: aCanvas 
		transformedCoordinatesUnitRectangle: transformedCoordinatesUnitRectangle
		yAxisX: yAxisX
		xAxisY: xAxisY 		
		xUnit: xUnit.
	self drawYAxisOn: aCanvas
		transformedCoordinatesUnitRectangle: transformedCoordinatesUnitRectangle
		xAxisY: xAxisY 
		yAxisX: yAxisX
		yUnit: yUnit.! !

!MGPlotMorph methodsFor: 'private morph drawing' stamp: 'mg 12/10/2013 10:42'!
drawOn: aCanvas
	(plotForm notNil and: [plotForm extent = self extent]) 
	ifTrue: 
	[
		aCanvas drawImage: plotForm at: self position
	]
	ifFalse:
	[
		self rememberToReplot.
		aCanvas fillRectangle: self bounds color: backgroundColor 
	].! !

!MGPlotMorph methodsFor: 'private form drawing' stamp: 'mg 12/11/2013 01:37'!
drawXAxisOn: aCanvas 
transformedCoordinatesUnitRectangle: transformedCoordinatesUnitRectangle
yAxisX: yAxisX
xAxisY: xAxisY 
xUnit: xUnit
	|maxXPoint|
	maxXPoint := (aCanvas extent x @ xAxisY) asIntegerPoint.
	aCanvas 
		line: (0@ xAxisY) asIntegerPoint
		to: maxXPoint
		color: axesColor.
	aCanvas
		drawPolygon: 
		{
			maxXPoint.
			maxXPoint + (-11 @ 3).
			maxXPoint + (-11 @ -3)
		}
		color: axesColor
		borderWidth: 1
		borderColor: axesColor.
	self
		drawXCoordinatesOn: aCanvas 
		transformedCoordinatesUnitRectangle: transformedCoordinatesUnitRectangle
		xAxisY: xAxisY 
		xStart: yAxisX - xUnit
		xStop: xUnit
		xStep: xUnit negated.
	self
		drawXCoordinatesOn: aCanvas 
		transformedCoordinatesUnitRectangle: transformedCoordinatesUnitRectangle
		xAxisY: xAxisY 
		xStart: yAxisX + xUnit
		xStop: maxXPoint x - xUnit
		xStep: xUnit.! !

!MGPlotMorph methodsFor: 'private form drawing' stamp: 'mg 12/11/2013 00:51'!
drawXCoordinatesOn: aCanvas 
transformedCoordinatesUnitRectangle: transformedCoordinatesUnitRectangle
xAxisY: xAxisY 
xStart: xStart
xStop: xStop
xStep: xStep
	|counter label|
	counter := xStep sign.
	xStart to: xStop by: xStep  do:
	[:x|
		aCanvas
			line: (x @ (xAxisY - 5)) asIntegerPoint
			to: (x @ (xAxisY + 5)) asIntegerPoint
			color: axesColor.
		label := self stringForXCoordinateNumbered: counter.
		aCanvas
			drawString: label
			at: (x @ xAxisY + (self xCorrectionForLabel: label)) asIntegerPoint
			font: nil
			color: axesColor.
		counter :=counter + xStep sign
	].! !

!MGPlotMorph methodsFor: 'private form drawing' stamp: 'mg 12/11/2013 01:40'!
drawYAxisOn: aCanvas 
transformedCoordinatesUnitRectangle: transformedCoordinatesUnitRectangle
xAxisY: xAxisY 
yAxisX: yAxisX
yUnit: yUnit
	|maxYPoint|
	maxYPoint := (yAxisX @ 0) asIntegerPoint.
	aCanvas 
		line: maxYPoint
		to: (yAxisX @ aCanvas extent y) asIntegerPoint
		color: axesColor.
	aCanvas
		drawPolygon: 
		{
			maxYPoint.
			maxYPoint + (3 @ 11).
			maxYPoint + (-3 @ 11)
		}
		color: axesColor
		borderWidth: 1
		borderColor: axesColor.
	self
		drawYCoordinatesOn: aCanvas 
		transformedCoordinatesUnitRectangle: transformedCoordinatesUnitRectangle
		yAxisX: yAxisX
		yStart: xAxisY - yUnit
		yStop: yUnit
		yStep: yUnit negated.
	self
		drawYCoordinatesOn: aCanvas 
		transformedCoordinatesUnitRectangle: transformedCoordinatesUnitRectangle
		yAxisX: yAxisX
		yStart: xAxisY + yUnit
		yStop: aCanvas extent y -yUnit
		yStep: yUnit.		! !

!MGPlotMorph methodsFor: 'private form drawing' stamp: 'mg 12/11/2013 00:50'!
drawYCoordinatesOn: aCanvas 
transformedCoordinatesUnitRectangle: transformedCoordinatesUnitRectangle
yAxisX: yAxisX
yStart: yStart
yStop: yStop
yStep: yStep
	|counter label|
	counter := yStep sign negated.
	yStart  to: yStop by: yStep do:
	[:y|
		aCanvas
			line: ((yAxisX - 5) @ y) asIntegerPoint
			to: ((yAxisX + 5)  @ y) asIntegerPoint
			color: axesColor.
		label := self stringForYCoordinateNumbered: counter.
		aCanvas
			drawString: label
			at: (yAxisX @ y + (self yCorrectionForLabel: label)) asIntegerPoint
			font: nil
			color: axesColor.
		counter :=counter  + yStep sign negated
	].! !

!MGPlotMorph methodsFor: 'private initialize-release' stamp: 'mg 12/11/2013 02:17'!
initialize
	super initialize.
	self extent: 400@400.
	plotRectangle := -10@ -10 corner: 10@10.
	coordinatesUnitRectangle := 0@0  corner: 1@1.
	relations := OrderedCollection new.
	backgroundColor := Color white.
	axesColor := Color black.
	extentSharedQueue := SharedQueue2 new.
	plotFormSharedQueue := SharedQueue2 new.
	xLabelsAbove := false.
	yLabelsLeft := true.
	plottingProcess := [self plottingLoop] fork.! !

!MGPlotMorph methodsFor: 'private form drawing' stamp: 'mg 4/25/2014 02:37'!
newFormForExtent: aPoint
	|canvas transform newPlotForm|
	newPlotForm := Form extent: aPoint depth: 32.
	newPlotForm fillColor: backgroundColor.
	canvas := newPlotForm getCanvas.
	transform := MGPlotToFormTransformation 
		plotRectangle: plotRectangle
		formRectangle: (0@0 extent: aPoint).
	self drawCoordinatesOn: canvas transformation: transform.		
	relations do:
	[:relation|
		relation drawOn: canvas transformation: transform.
	].
	^ newPlotForm.
	
	
	! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/10/2013 11:01'!
originPoint
	"Return the origin of the plot in the plotting space."
	^coordinatesUnitRectangle origin! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/10/2013 11:01'!
originPoint: aPoint
	"Set the origin of the plot in the plotting space."
	coordinatesUnitRectangle := aPoint extent: (coordinatesUnitRectangle extent).
	self rememberToReplot! !

!MGPlotMorph methodsFor: 'private processes' stamp: 'mg 12/6/2013 22:10'!
plottingLoop
	|extent|
	[
		extent := extentSharedQueue nextOrNil.
		extent ifNotNil:
		[
			plotFormSharedQueue nextPut: (self newFormForExtent: extent)
		].
		plottingProcess suspend
	] repeat.! !

!MGPlotMorph methodsFor: 'private initialize-release' stamp: 'mg 12/11/2013 02:19'!
privateDelete
	|process|
	super privateDelete.
	process := plottingProcess.
	plottingProcess := nil.
	process ifNotNil: [process terminate].! !

!MGPlotMorph methodsFor: 'relations' stamp: 'mg 12/7/2013 13:50'!
relations
	"Return the ordered collection of MGRelations"
	^relations asArray! !

!MGPlotMorph methodsFor: 'plotting' stamp: 'mg 12/10/2013 11:17'!
rememberToReplot
	"Remember to redraw the plot, when all points get calculated"
	extentSharedQueue isEmpty ifTrue: [extentSharedQueue nextPut: self extent].
	
	! !

!MGPlotMorph methodsFor: 'relations' stamp: 'mg 12/7/2013 13:50'!
removeAllRelations
	"Remove all MGRelations from a plot."
	relations do:
	[:relation|
		relation removeDependent: self
	].
	relations removeAll.
	self rememberToReplot.! !

!MGPlotMorph methodsFor: 'relations' stamp: 'mg 12/7/2013 13:51'!
removeRelation: anMGRelation
	"Remove an MGRelation from the plot."
	relations remove: anMGRelation.
	anMGRelation removeDependent: self.
	self rememberToReplot.! !

!MGPlotMorph methodsFor: 'private stepping and presenter' stamp: 'mg 12/11/2013 01:44'!
step
	plotFormSharedQueue isEmpty
	ifFalse:
	[
		[plotFormSharedQueue isEmpty] 
		whileFalse:
		[plotForm := plotFormSharedQueue next].
		self changed
	].
	plottingProcess ifNotNil:[plottingProcess resume]! !

!MGPlotMorph methodsFor: 'private testing' stamp: 'mg 12/10/2013 12:13'!
stepTime
	^100! !

!MGPlotMorph methodsFor: 'private form drawing' stamp: 'mg 12/11/2013 00:13'!
stringForXCoordinateNumbered: anInteger
	|number|
	number := (anInteger * coordinatesUnitRectangle width + coordinatesUnitRectangle origin x).
	xLabelsMaxDecimalPlaces 
	ifNil: [^number asString]
	ifNotNil: [^number printShowingMaxDecimalPlaces: xLabelsMaxDecimalPlaces]! !

!MGPlotMorph methodsFor: 'private form drawing' stamp: 'mg 12/11/2013 00:13'!
stringForYCoordinateNumbered: anInteger
	|number|
	number := (anInteger * coordinatesUnitRectangle height + coordinatesUnitRectangle origin y).
	yLabelsMaxDecimalPlaces 
	ifNil: [^number asString]
	ifNotNil: [^number printShowingMaxDecimalPlaces: yLabelsMaxDecimalPlaces]
	! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/7/2013 13:55'!
units
	"Return a Point which shows the units on the x and y axes."
	^ coordinatesUnitRectangle extent.! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/7/2013 13:55'!
units: aPoint
	"Return units for both x and y axes."
	coordinatesUnitRectangle := coordinatesUnitRectangle origin extent: aPoint.
	self rememberToReplot.! !

!MGPlotMorph methodsFor: 'private updating' stamp: 'mg 12/1/2013 16:02'!
update: aParameter
	self rememberToReplot! !

!MGPlotMorph methodsFor: 'private form drawing' stamp: 'mg 12/11/2013 01:22'!
xCorrectionForLabel: aString
	|font width height lastWidth|
	font := TextStyle default defaultFont.
	width :=  font widthOfString: aString.
	height := font height.
	lastWidth := font widthOf: aString last.
	^(width negated + (lastWidth // 2) + 2)
	@
	(
		xLabelsAbove 
			ifTrue: [height negated - 6]
			ifFalse: [7]
	)	
	
	! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/11/2013 00:19'!
xLabelsAbove
	"Make x coordinate labels to be above the axis."
	xLabelsAbove := true.
	self rememberToReplot.! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/11/2013 00:19'!
xLabelsBelow
	"Make x coordinate labels to be below the axis."
	xLabelsAbove := false.
	self rememberToReplot.! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/11/2013 00:07'!
xLabelsMaxDecimalPlaces
	"Get the number of decimal places while displaying x coordinate labels"
	^ xLabelsMaxDecimalPlaces! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/11/2013 00:19'!
xLabelsMaxDecimalPlaces: anInteger
	"Set the number of decimal places while displaying x coordinate labels"
	xLabelsMaxDecimalPlaces := anInteger.
	self rememberToReplot.! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/10/2013 10:51'!
xMax
	"Return the maximum value on the x axis"
	^ plotRectangle right! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/10/2013 10:51'!
xMax: aNumber
	"Set the maximum value on the x axis"
	plotRectangle := plotRectangle right: aNumber.
	self rememberToReplot.! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/10/2013 10:51'!
xMin
	"Return the minimum value on the x axis"
	^ plotRectangle left! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/10/2013 10:52'!
xMin: aNumber
	"Set the minimum value on the x axis"
	plotRectangle := plotRectangle left: aNumber.
	self rememberToReplot.	! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/10/2013 10:57'!
xRange
	"Return the range of the x axis"
	^ plotRectangle left @ plotRectangle right! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 3/26/2014 11:48'!
xRange: aPoint
	"Set the range of the x axis"
	plotRectangle := (plotRectangle left: aPoint x) right: aPoint y.
	self rememberToReplot.! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/7/2013 13:55'!
xUnit
	"Return the unit of the x axis."
	^coordinatesUnitRectangle width! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/7/2013 13:56'!
xUnit: aNumber
	"Set the unit of the x axis."
	coordinatesUnitRectangle := coordinatesUnitRectangle origin 
		extent: aNumber @ coordinatesUnitRectangle extent y.
	self rememberToReplot.! !

!MGPlotMorph methodsFor: 'private form drawing' stamp: 'mg 12/11/2013 01:18'!
yCorrectionForLabel: aString
	|font width height minusWidth|
	font := TextStyle default defaultFont.
	width :=  font widthOfString: aString.
	height := font height.
	minusWidth := font widthOf: $-.
	^(
		yLabelsLeft
		ifTrue: [width negated - 6]
		ifFalse: 
		[
			aString first = $- 
			ifTrue: [8]
			ifFalse: [minusWidth + 8]
		]
	)
	@	
	(height negated // 2)
! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/11/2013 00:20'!
yLabelsLeft
	"Make y coordinate labels to be to the left of the axis."
	yLabelsLeft := true.
	self rememberToReplot.	! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/11/2013 00:08'!
yLabelsMaxDecimalPlaces
	"Get the number of decimal places while displaying y coordinates"
	^ yLabelsMaxDecimalPlaces! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/11/2013 00:20'!
yLabelsMaxDecimalPlaces: anInteger
	"Set the number of decimal places while displaying y coordinates"
	yLabelsMaxDecimalPlaces := anInteger.
	self rememberToReplot.	! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/11/2013 00:20'!
yLabelsRight
	"Make y coordinate labels to be to the right of the axis."
	yLabelsLeft := false.
	self rememberToReplot.	! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/10/2013 10:51'!
yMax
	"Return the maximum value on the y axis"
	^ plotRectangle top negated! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/10/2013 10:51'!
yMax: aNumber
	"Set the maximum value on the y axis"
	plotRectangle := plotRectangle top: aNumber negated.
	self rememberToReplot.	! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/10/2013 10:52'!
yMin
	"Return the minimum value on the y axis"
	^ plotRectangle bottom negated! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/10/2013 10:52'!
yMin: aNumber
	"Set the maximum value on the y axis"
	plotRectangle := plotRectangle bottom: aNumber negated.
	self rememberToReplot.	! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/10/2013 11:00'!
yRange
	"Return the range of the y axis"
	^ plotRectangle bottom negated @ plotRectangle top negated! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 3/26/2014 11:49'!
yRange: aPoint
	"Set the range of the y axis"
	plotRectangle := (plotRectangle bottom: aPoint x negated) top: aPoint y negated.
	self rememberToReplot.	! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/7/2013 13:56'!
yUnit
	"Return the unit of the y axis."
	^coordinatesUnitRectangle height! !

!MGPlotMorph methodsFor: 'coordinate system' stamp: 'mg 12/7/2013 13:56'!
yUnit: aNumber
	"Set the unit of the y axis."
	coordinatesUnitRectangle := coordinatesUnitRectangle origin 
		extent: coordinatesUnitRectangle extent x @ aNumber.
	self rememberToReplot.! !

Object subclass: #MGPlotToFormTransformation
	instanceVariableNames: 'toPlotMatrix toFormMatrix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGPlotToFormTransformation commentStamp: 'mg 12/7/2013 14:03' prior: 0!
A MGPlotToFormTransformation is a class for performing the transformation from the plot space to the space of the pixels of the form which is displayed inside an MGPlotMorph.

Instance Variables
	toFormMatrix:		<Matrix>
	toPlotMatrix:		<Matrix>

toFormMatrix
	- a matrix for transforming from the plot to the form

toPlotMatrix
	- a matrix for transforming from the form to the plot
!

!MGPlotToFormTransformation class methodsFor: 'instance creation' stamp: 'mg 12/7/2013 13:58'!
plotRectangle: plotRectangle formRectangle: formRectangle
	"Return a new transformation between plotRectangle and formRectangle."
	^ self new plotRectangle: plotRectangle formRectangle: formRectangle! !

!MGPlotToFormTransformation methodsFor: 'private initialize-release' stamp: 'mg 12/1/2013 02:07'!
calculateToFormMatrixUsingPlotRectangle: plotRectangle formRectangle: formRectangle
	|pm0 pm1 pm2 pm3|
	pm0 := Matrix
		rows: 3 
		columns: 3 
		contents:
		{
			1.  0. 0.
			0. -1. 0.
			0.  0. 1
		}.
	pm1 := Matrix 
		rows: 3 
		columns: 3 
		contents:
		{
			1. 0. plotRectangle origin x negated.
			0. 1. plotRectangle origin y negated.
			0. 0. 1
		}.
	pm2 := Matrix 
		rows: 3 
		columns: 3 
		contents:
		{
			(formRectangle corner x - formRectangle origin x) 
			/ (plotRectangle corner x - plotRectangle origin x). 0. 0.
			0. (formRectangle corner y - formRectangle origin y) 
			/ (plotRectangle corner y - plotRectangle origin y). 0.
			0. 0. 1
		}.		
	pm3 := Matrix 
		rows: 3 
		columns: 3 
		contents:
		{
			1. 0. formRectangle origin x.
			0. 1. formRectangle origin y.
			0. 0. 1
		}.
	toFormMatrix := (pm3 +* pm2 +*  pm1 +* pm0) atRows: 1 to: 2 columns: 1 to: 3.! !

!MGPlotToFormTransformation methodsFor: 'private initialize-release' stamp: 'mg 12/1/2013 02:05'!
calculateToPlotMatrixUsingPlotRectangle: plotRectangle formRectangle: formRectangle
	|mp0 mp1 mp2 mp3|
	mp0 := Matrix
		rows: 3 
		columns: 3 
		contents:
		{
			1.  0. 0.
			0. -1. 0.
			0.  0. 1
		}.
	mp1 := Matrix 
		rows: 3 
		columns: 3 
		contents:
		{
			1. 0. formRectangle origin x negated.
			0. 1. formRectangle origin y negated.
			0. 0. 1
		}.
	mp2 := Matrix 
		rows: 3 
		columns: 3 
		contents:
		{
			(plotRectangle corner x - plotRectangle origin x) 
			/ (formRectangle corner x - formRectangle origin x). 0. 0.
			0. (plotRectangle corner y - plotRectangle origin y) 
			/ (formRectangle corner y - formRectangle origin y). 0.
			0. 0. 1
		}.		
	mp3 := Matrix 
		rows: 3 
		columns: 3 
		contents:
		{
			1. 0. plotRectangle origin x.
			0. 1. plotRectangle origin y.
			0. 0. 1
		}.
	toPlotMatrix := (mp0 +* mp3 +* mp2 +*  mp1) atRows: 1 to: 2 columns: 1 to: 3.! !

!MGPlotToFormTransformation methodsFor: 'private initialize-release' stamp: 'mg 12/1/2013 02:08'!
plotRectangle: plotRectangle formRectangle: formRectangle
	self calculateToFormMatrixUsingPlotRectangle: plotRectangle formRectangle: formRectangle.
	self calculateToPlotMatrixUsingPlotRectangle: plotRectangle formRectangle: formRectangle.	
! !

!MGPlotToFormTransformation methodsFor: 'transformation' stamp: 'mg 12/7/2013 13:59'!
toFormPoint: aPoint
	"Apply the transformation from a plot Point to a form point and return the form Point."
	|array|
	array := toFormMatrix +* {aPoint x. aPoint y. 1}.
	^ array first @ array last! !

!MGPlotToFormTransformation methodsFor: 'transformation' stamp: 'mg 12/7/2013 14:00'!
toFormPoints: aCollection
	"Apply the transformation from plot Points to form Points and return the form Points."
	^aCollection collect: [:aPoint| self toFormPoint: aPoint].! !

!MGPlotToFormTransformation methodsFor: 'transformation' stamp: 'mg 12/7/2013 14:01'!
toFormRectangle: aRectangle
	"Apply the transformation from plot Rectangle to form Rectangle and return the form Rectangle."
	^(self toFormPoint: aRectangle origin) corner: (self toFormPoint: aRectangle corner)! !

!MGPlotToFormTransformation methodsFor: 'transformation' stamp: 'mg 12/7/2013 13:59'!
toPlotPoint: aPoint
	"Apply the transformation from a form Point to a plot point and return the plot Point."
	|array|
	array := toPlotMatrix +* {aPoint x. aPoint y. 1}.
	^ array first @ array last! !

!MGPlotToFormTransformation methodsFor: 'transformation' stamp: 'mg 12/7/2013 14:00'!
toPlotPoints: aCollection
	"Apply the transformation from form Points to plot Points and return the plot Points."
	^aCollection collect: [:aPoint| self toPlotPoint: aPoint].! !

!MGPlotToFormTransformation methodsFor: 'transformation' stamp: 'mg 12/7/2013 14:01'!
toPlotRectangle: aRectangle
	"Apply the transformation from form Rectangle to plot Rectangle and return the plot Rectangle."
	^(self toPlotPoint: aRectangle origin) corner: (self toPlotPoint: aRectangle corner)! !

Object subclass: #MGPoint
	instanceVariableNames: 'point pointIcon'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGPoint commentStamp: 'mg 12/7/2013 14:07' prior: 0!
A MGPoint is a class which keeps both a point and its symbols as its private variables. It is used in while using the MGRelationOnCollectionWithIcons.

Instance Variables
	point:	<Point>
	pointIcon:	<MGPointIcon>

point
	- a Point

pointIcon
	- an icon of the point
!

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:27'!
bigCrossPoint: aPoint
	^self new point: aPoint; pointIcon: MGCrossPointIcon big! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:29'!
bigCrossPoint: aPoint color: aColor
	^self new point: aPoint; pointIcon: (MGCrossPointIcon bigWithColor: aColor)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:27'!
bigRectanglePoint: aPoint
	^self new point: aPoint; pointIcon: MGRectanglePointIcon big! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:29'!
bigRectanglePoint: aPoint color: aColor
	^self new point: aPoint; pointIcon: (MGRectanglePointIcon bigWithColor: aColor)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:24'!
crossPoint: aPoint
	^self new point: aPoint; pointIcon: MGCrossPointIcon new! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 10:51'!
crossPoint: aPoint color: aColor
	^self new point: aPoint; pointIcon: (MGCrossPointIcon color: aColor)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:30'!
crossPoint: aPoint1 color: aColor extent: aPoint2
	^self new point: aPoint1; pointIcon: (MGCrossPointIcon color: aColor extent: aPoint2)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:37'!
crossPoint: aPoint1 extent: aPoint2
	^self new point: aPoint1; pointIcon: (MGCrossPointIcon extent: aPoint2)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:24'!
dotPoint: aPoint
	^self new point: aPoint; pointIcon: MGDotPointIcon new! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 10:51'!
dotPoint: aPoint color: aColor
	^self new point: aPoint; pointIcon: (MGDotPointIcon color: aColor)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:26'!
dotPoint: aPoint1 color: aColor extent: aPoint2
	^self new point: aPoint1; pointIcon: (MGDotPointIcon color: aColor; extent: aPoint2)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:27'!
mediumCrossPoint: aPoint
	^self new point: aPoint; pointIcon: MGCrossPointIcon medium! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:28'!
mediumCrossPoint: aPoint color: aColor
	^self new point: aPoint; pointIcon: (MGCrossPointIcon mediumWithColor: aColor)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:26'!
mediumRectanglePoint: aPoint
	^self new point: aPoint; pointIcon: MGRectanglePointIcon medium! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:29'!
mediumRectanglePoint: aPoint color: aColor
	^self new point: aPoint; pointIcon: (MGRectanglePointIcon mediumWithColor: aColor)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 10:43'!
point: aPoint pointIcon: aPointIcon
	^self new point: aPoint; pointIcon: aPointIcon ! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:24'!
rectanglePoint: aPoint
	^self new point: aPoint; pointIcon: MGRectanglePointIcon new! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:13'!
rectanglePoint: aPoint color: aColor
	^self new point: aPoint; pointIcon: (MGRectanglePointIcon color: aColor)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:30'!
rectanglePoint: aPoint1 color: aColor extent: aPoint2
	^self new point: aPoint1; pointIcon: (MGRectanglePointIcon color: aColor extent: aPoint2)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:37'!
rectanglePoint: aPoint1 extent: aPoint2
	^self new point: aPoint1; pointIcon: (MGRectanglePointIcon extent: aPoint2)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:27'!
smallCrossPoint: aPoint
	^self new point: aPoint; pointIcon: MGCrossPointIcon small! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:28'!
smallCrossPoint: aPoint color: aColor
	^self new point: aPoint; pointIcon: (MGCrossPointIcon smallWithColor: aColor)! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:26'!
smallRectanglePoint: aPoint
	^self new point: aPoint; pointIcon: MGRectanglePointIcon small! !

!MGPoint class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:29'!
smallRectanglePoint: aPoint color: aColor
	^self new point: aPoint; pointIcon: (MGRectanglePointIcon smallWithColor: aColor)! !

!MGPoint methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:05'!
point
	"Return a point."
	^ point! !

!MGPoint methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:05'!
point: aPoint
	"Set a point."
	point := aPoint! !

!MGPoint methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:06'!
pointIcon
	"Return a MGPointIcon of the point."
	^ pointIcon! !

!MGPoint methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:06'!
pointIcon: anMGPointIcon
	"Set a MGPointIcon of the point."
	pointIcon := anMGPointIcon! !

Object subclass: #MGPointIcon
	instanceVariableNames: 'color extent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGPointIcon commentStamp: 'mg 12/7/2013 14:09' prior: 0!
A MGPointIcon is a class of icons used for displaying points inside an MGPlotMorph. You can subclass this class
to add additional ways of displaying points inside the plot.

Instance Variables
	color:	<Color>
	extent:	<Point>

color
	- the color of the point

extent
	- the extent of the icon (a user can resize the icon by setting this)
!

MGPointIcon subclass: #MGCrossPointIcon
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGCrossPointIcon commentStamp: 'mg 12/7/2013 14:15' prior: 0!
A MGCrossPointIcon is a cross icon. For details see the MGPointIcon comment.

Instance Variables
!

!MGCrossPointIcon methodsFor: 'private drawing' stamp: 'mg 12/2/2013 11:23'!
drawOn: aCanvas at: aPoint
	aCanvas
		line: (aPoint - (extent / 2)) asIntegerPoint
		to: (aPoint + (extent / 2)) asIntegerPoint
		color: color.
	aCanvas
		line: (aPoint - (extent leftRotated / 2)) asIntegerPoint
		to: (aPoint + (extent leftRotated / 2)) asIntegerPoint
		color: color.		! !

MGPointIcon subclass: #MGDotPointIcon
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGDotPointIcon commentStamp: 'mg 12/7/2013 14:15' prior: 0!
A MGDotPointIcon is a small one pixel dot icon. For details see the MGPointIcon comment.

Instance Variables
!

!MGDotPointIcon methodsFor: 'private drawing' stamp: 'mg 12/1/2013 15:10'!
drawOn: aCanvas at: aPoint
	aCanvas
		point: aPoint asIntegerPoint
		color: color! !

!MGPointIcon class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:28'!
big
	^ self new extent: 8@8! !

!MGPointIcon class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:28'!
bigWithColor: aColor
	^ self new extent: 8@8; color: aColor! !

!MGPointIcon class methodsFor: 'instance creation' stamp: 'mg 12/1/2013 15:09'!
color: aColor
	^ self new color: aColor! !

!MGPointIcon class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:17'!
color: aColor extent: aPoint
	^ self new color: aColor; extent: aPoint! !

!MGPointIcon class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:17'!
extent: aPoint
	^ self new extent: aPoint! !

!MGPointIcon class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:20'!
medium
	^ self new extent: 4@4! !

!MGPointIcon class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:19'!
mediumWithColor: aColor
	^ self new extent: 4@4; color: aColor! !

!MGPointIcon class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:20'!
small
	^ self new extent: 1@1! !

!MGPointIcon class methodsFor: 'instance creation' stamp: 'mg 12/2/2013 11:35'!
smallWithColor: aColor
	^ self new extent: 2@2; color: aColor! !

!MGPointIcon methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:10'!
color
	"Return the color of the icon."
	^ color ! !

!MGPointIcon methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:10'!
color: aColor
	"Set the color of the icon."
	color := aColor! !

!MGPointIcon methodsFor: 'private drawing' stamp: 'mg 12/1/2013 15:02'!
drawOn: aCanvas at: aPoint
	self subclassResponsibility! !

!MGPointIcon methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:11'!
extent
	"Return the extent of the icon as a Point."
	^ extent! !

!MGPointIcon methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:10'!
extent: aPoint
	"Return the extent of the icon."
	extent := aPoint! !

!MGPointIcon methodsFor: 'private initialize-release' stamp: 'mg 12/7/2013 15:28'!
initialize
	super initialize.
	color := Color black.
	extent := 5@5.! !

MGPointIcon subclass: #MGRectanglePointIcon
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGRectanglePointIcon commentStamp: 'mg 12/7/2013 14:15' prior: 0!
A MGRectanglePointIcon is a rectangle icon. For details see the MGPointIcon comment.

Instance Variables
!

!MGRectanglePointIcon methodsFor: 'private drawing' stamp: 'mg 12/2/2013 11:45'!
drawOn: aCanvas at: aPoint
	aCanvas fillRectangle: (Rectangle center: aPoint asIntegerPoint extent: extent asIntegerPoint) color: color.! !

Object subclass: #MGRelation
	instanceVariableNames: 'color isContinuous pointIcon'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGRelation commentStamp: 'mg 12/7/2013 14:33' prior: 0!
A MGRelation is an abstract class of relations displayable inside an MGPlotMorph.

Instance Variables
	isContinuous:		<Boolean>
	color:	<Color>
	pointIcon:	<MGPointIcon>

isContinuous
	- if it is false the relations is drawn using separate points, if true it's drawn using connected lines .

color
	- the color of the relation points/lines

pointIcon
	- the default icon of the relation points
!

!MGRelation methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:33'!
color
	"Return the color of the relation."
	^color! !

!MGRelation methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:33'!
color: aColor
	"Set the color of the relation."
	color := aColor.
	pointIcon color: aColor.
	self changed! !

!MGRelation methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:34'!
continuous
	"Use connected lines for drawing the relation."
	isContinuous := true.
	self changed.! !

!MGRelation methodsFor: 'private drawing' stamp: 'mg 12/1/2013 02:53'!
drawOn: aCanvas transformation: aPlotToFormTransformation
	isContinuous 
	ifTrue:
	[
		self drawContinuousOn: aCanvas transformation: aPlotToFormTransformation
	]
	ifFalse:
	[
		self drawNotContinuousOn: aCanvas transformation: aPlotToFormTransformation	
	]! !

!MGRelation methodsFor: 'private initialize-release' stamp: 'mg 12/7/2013 14:24'!
initialize
	super initialize.
	color := Color black.
	isContinuous := false.
	pointIcon := MGDotPointIcon color: color.! !

!MGRelation methodsFor: 'testing' stamp: 'mg 12/7/2013 14:35'!
isContinuous
	"Return true if the relation is drawn by using connected lines, false if by using separate points."
	^ isContinuous! !

!MGRelation methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:34'!
notContinuous
	"Use separate points for drawing the relation."
	isContinuous := false.
	self changed.! !

!MGRelation methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:34'!
pointIcon
	"Return the default icon of the points."
	^ pointIcon! !

!MGRelation methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:34'!
pointIcon: aPointIcon
	"Set the default icon of the points."
	pointIcon := aPointIcon.
	self changed.! !

MGRelation subclass: #MGRelationOnBlock
	instanceVariableNames: 'block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGRelationOnBlock commentStamp: 'mg 12/7/2013 15:37' prior: 0!
A MGRelationOnBlock is an abstract class for plotting by using data from a block.

Instance Variables
	block:		<BlockClosure>

block
	- the block used for plotting
!

MGRelationOnBlock subclass: #MGRelationOn1DBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGRelationOn1DBlock commentStamp: 'mg 12/7/2013 14:38' prior: 0!
A MGRelationOn1DBlock is an abstract class used for plotting with a block, which takes one argument.

Instance Variables
!

!MGRelationOn1DBlock methodsFor: 'private initialize-release' stamp: 'mg 12/2/2013 11:52'!
initialize
	super initialize.
	block := [:a|a].! !

!MGRelationOn1DBlock methodsFor: 'private calculating block values' stamp: 'mg 12/2/2013 11:51'!
valuesFor: anObject
		|values|
		values := block value: anObject.
		true caseOf:
		{
			[values isNil] -> [^{}].
			[values isCollection] -> [^values]
		}
		otherwise: [^{values}].
		! !

MGRelationOn1DBlock subclass: #MGRelationOnLeftToRightBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGRelationOnLeftToRightBlock commentStamp: 'mg 12/7/2013 15:35' prior: 0!
A MGRelationOnLeftToRightBlock is a class used for plotting with a block. The block takes one argument which is an x coordinate. It returns either a y coordinate or a collection of y coordinates.

Example:
	|mgPlotMorph mgRelationOnLeftToRightBlock|
	mgPlotMorph := MGPlotMorph new.
	mgPlotMorph openCenteredInWorld.
	mgRelationOnLeftToRightBlock := MGRelationOnLeftToRightBlock new 
		block: [:x|x squared]; 
		continuous.
	mgPlotMorph addRelation: mgRelationOnLeftToRightBlock.

Instance Variables
!

!MGRelationOnLeftToRightBlock methodsFor: 'private drawing' stamp: 'mg 12/7/2013 14:25'!
drawContinuousOn: aCanvas transformation: aPlotToFormTransformation
	|xValue currentValues previousValues|
	previousValues := (self valuesFor: ((aPlotToFormTransformation toPlotPoint: 0 @ 0) x)).
	
	1 to: aCanvas extent x do:
	[:xVal|
		xValue := (aPlotToFormTransformation toPlotPoint: xVal @ 0) x.
		currentValues := self valuesFor: xValue.
		currentValues with: previousValues do:
		[:currentYValue :previousYValue|
			aCanvas
				line: (aPlotToFormTransformation toFormPoint: xValue @ previousYValue) asIntegerPoint
				to: (aPlotToFormTransformation toFormPoint: xValue @ currentYValue) asIntegerPoint
				color: color.
		].
		previousValues := currentValues.
	]! !

!MGRelationOnLeftToRightBlock methodsFor: 'private drawing' stamp: 'mg 12/2/2013 12:04'!
drawNotContinuousOn: aCanvas transformation: aPlotToFormTransformation
	|xValue|
	0 to: aCanvas extent x do:
	[:xVal|
		xValue := (aPlotToFormTransformation toPlotPoint: xVal @ 0) x.
		(self valuesFor: xValue) do:
		[:yValue|
			pointIcon 
				drawOn: aCanvas
				at: (aPlotToFormTransformation toFormPoint: xValue @ yValue)
		]
	]! !

MGRelationOn1DBlock subclass: #MGRelationOnTopToBottomBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGRelationOnTopToBottomBlock commentStamp: 'mg 12/7/2013 15:36' prior: 0!
A MGRelationOnTopToBottomBlock is a class used for plotting with a block. The block takes one argument which is an y coordinate. It returns either an x coordinate or a collection of x coordinates.

Example:
	|mgPlotMorph mgRelationOnTopToBottomBlock|
	mgPlotMorph := MGPlotMorph new.
	mgPlotMorph openCenteredInWorld.
	mgRelationOnTopToBottomBlock := MGRelationOnTopToBottomBlock new 
		block: [:y|y squared]; 
		continuous.
	mgPlotMorph addRelation: mgRelationOnTopToBottomBlock.

Instance Variables
!

!MGRelationOnTopToBottomBlock methodsFor: 'private drawing' stamp: 'mg 12/7/2013 15:14'!
drawContinuousOn: aCanvas transformation: aPlotToFormTransformation
	|yValue currentValues previousValues|
	previousValues := (self valuesFor: ((aPlotToFormTransformation toPlotPoint: 0 @ 0) y)).
	
	1 to: aCanvas extent y do:
	[:yVal|
		yValue := (aPlotToFormTransformation toPlotPoint: 0 @ yVal) y.
		currentValues := self valuesFor: yValue.
		currentValues with: previousValues do:
		[:currentXValue :previousXValue|
			aCanvas
				line: (aPlotToFormTransformation toFormPoint: previousXValue @ yValue) asIntegerPoint
				to: (aPlotToFormTransformation toFormPoint: currentXValue @ yValue) asIntegerPoint
				color: color.		
		].
		previousValues := currentValues.	
	]! !

!MGRelationOnTopToBottomBlock methodsFor: 'private drawing' stamp: 'mg 12/2/2013 12:03'!
drawNotContinuousOn: aCanvas transformation: aPlotToFormTransformation
	|yValue|
	0 to: aCanvas extent y do:
	[:yVal|
		yValue := (aPlotToFormTransformation toPlotPoint: 0 @ yVal) y.
		(self valuesFor: yValue) do:
		[:xValue|
			pointIcon 
				drawOn: aCanvas
				at: (aPlotToFormTransformation toFormPoint: xValue @ yValue)
		]
	]! !

MGRelationOnBlock subclass: #MGRelationOn2DBlock
	instanceVariableNames: 'minimalStep'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGRelationOn2DBlock commentStamp: 'mg 12/7/2013 20:45' prior: 0!
A MGRelationOn2DBlock is aclass used for plotting with a block, which takes two arguments, which are two coordinates x and y. The block returns a value between 0 and 1, which shows the 1-alpha of the color used 
for plotting. It means it is really a 3D plot, but who cares :-P

Example:
	|mgPlotMorph mgRelationOn2DBlock|
	mgPlotMorph := MGPlotMorph new.
	mgPlotMorph openCenteredInWorld.
	mgRelationOn2DBlock := MGRelationOn2DBlock new
		color: Color blue;
		minimalStep: 0.1;
		block: [:x :y|(x@y) r ln min: 1.0 max: 0.0].
	mgPlotMorph addRelation: mgRelationOn2DBlock.


Calculating the value  of a plot for each pixel takes much time. For this reason the minimalStep is defined.
It describes the minimal step you want to take between two points used for calculations in the plot space.
So if you set minimalStep to 1, it means the calculations will be done only for points: (0@0, 1@0, 0@1, 1@1, ...),
(unless the morph size is too small, in this case the calculations will be done for each pixel).
In general:
If calculations take too long use a bigger number for minimalStep.
The default is 0 (no minimal step).

Instance Variables
	minimalStep:		<Point>

minimalStep
	- minimal step between pixels used for calculations in the plot space (separately for x and y)
!

!MGRelationOn2DBlock methodsFor: 'private drawing' stamp: 'mg 12/7/2013 20:46'!
drawOn: aCanvas transformation: aPlotToFormTransformation
	|formXStep formYStep value zeroPoint zeroPointX zeroPointY xVersor yVersor xVersorX xVersorY yVersorX yVersorY|
	"Z optymalizacjami"
	formXStep := 1.
	formYStep := 1.
	zeroPoint := aPlotToFormTransformation toPlotPoint: 0@0.
	xVersor := (aPlotToFormTransformation toPlotPoint: 1@0) - zeroPoint.
	yVersor := (aPlotToFormTransformation toPlotPoint: 0@1) - zeroPoint.
	xVersor r < minimalStep x
	ifTrue: 
	[	
		formXStep := (
			aPlotToFormTransformation 
				toFormPoint: zeroPoint + (xVersor normalized * minimalStep x)
		) r ceiling.			
	].
	yVersor r < minimalStep y
	ifTrue: 
	[			
		formYStep := (
			aPlotToFormTransformation 
				toFormPoint: zeroPoint + (yVersor normalized * minimalStep y)
		) r ceiling.			
	].
	
	xVersorX := xVersor x.
	xVersorY := xVersor y.	
	zeroPointX := zeroPoint x.
	zeroPointY := zeroPoint y.	
	yVersorX := yVersor x.
	yVersorY := yVersor y.	
	0 to: aCanvas extent x  by: formXStep do:
	[:x|
		0 to: aCanvas extent y by: formYStep do:
		[:y||xSrodka ySrodka|
			xSrodka := x + (formXStep // 2).
			ySrodka := y + (formYStep // 2).
			value := block 
				value: zeroPointX + (xSrodka*xVersorX) + (ySrodka*yVersorX)
				value: zeroPointY + (xSrodka*xVersorY) + (ySrodka*yVersorY).
			aCanvas
				fillRectangle:
				(x@y extent: formXStep @ formYStep)
				color: (color alpha: value).
		]
	]! !

!MGRelationOn2DBlock methodsFor: 'private initialize-release' stamp: 'mg 12/7/2013 20:45'!
initialize
	super initialize.
	block := [:aPoint|Color transparent].
	minimalStep := 0@0.! !

!MGRelationOn2DBlock methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:53'!
minimalStep
	"Return the minimal step between calculations in the plot space. See class comment."
	^ minimalStep! !

!MGRelationOn2DBlock methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:53'!
minimalStep: anObject
	"Set the minimal step between calculations in the plot space. See class comment."
	minimalStep := anObject.
	self changed.
	! !

!MGRelationOnBlock methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:37'!
block
	"Return the block used to calculate plot values."
	^block! !

!MGRelationOnBlock methodsFor: 'accessing' stamp: 'mg 12/7/2013 14:37'!
block: aBlock
	"Set the block used to calculate plot values."
	block := aBlock.
	self changed.! !

MGRelation subclass: #MGRelationOnCollection
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGRelationOnCollection commentStamp: 'mg 12/7/2013 15:32' prior: 0!
A MGRelationOnCollection is a class used for plotting any seuenceable collection of points. If used in continuous mode it will draw a line between each pair of point in the order of the collection.

Example:
	|mgPlotMorph mgRelationOnCollection|
	mgPlotMorph := MGPlotMorph new.
	mgPlotMorph openCenteredInWorld.
	mgRelationOnCollection := MGRelationOnCollection new 
		continuous;
		collection: {0@0. 1@1. 0@1. 1@0}.
	mgPlotMorph addRelation: mgRelationOnCollection.

Instance Variables
	collection:		<SequenceableCollection of Points>

collection
	- a sequenceable collection of Points
!

!MGRelationOnCollection methodsFor: 'accessing' stamp: 'mg 12/7/2013 15:00'!
collection
	"Returned the collection of Points to plot."
	^ collection! !

!MGRelationOnCollection methodsFor: 'accessing' stamp: 'mg 12/7/2013 15:00'!
collection: aCollection
	"Set the collection of Points to plot."
	collection := aCollection.
	self changed.! !

!MGRelationOnCollection methodsFor: 'private drawing' stamp: 'mg 12/7/2013 14:25'!
drawContinuousOn: aCanvas transformation: aPlotToFormTransformation
	collection overlappingPairsDo:
	[:firstPoint :secondPoint|
		aCanvas 
			line: (aPlotToFormTransformation toFormPoint: firstPoint) asIntegerPoint
			to: (aPlotToFormTransformation toFormPoint: secondPoint) asIntegerPoint
			color: color
	]! !

!MGRelationOnCollection methodsFor: 'private drawing' stamp: 'mg 12/2/2013 10:41'!
drawNotContinuousOn: aCanvas transformation: aPlotToFormTransformation
	collection do:
	[:point||transformedPoint|
		transformedPoint := (aPlotToFormTransformation toFormPoint: point).
		pointIcon drawOn: aCanvas at: transformedPoint.
	]! !

!MGRelationOnCollection methodsFor: 'private initialize-release' stamp: 'mg 12/2/2013 10:48'!
initialize
	super initialize.
	collection := OrderedCollection new.! !

MGRelationOnCollection subclass: #MGRelationOnCollectionWithIcons
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MGPlotting'!

!MGRelationOnCollectionWithIcons commentStamp: 'mg 12/7/2013 15:31' prior: 0!
A MGRelationOnCollectionWithIcons is a class similar to its superclass, but it uses a collection of MGPoints,
which allow you to use a different MGPointIcon for each point.

Example:
	|mgPlotMorph mgRelationOnCollectionWithIcons|
	mgPlotMorph := MGPlotMorph new.
	mgPlotMorph openCenteredInWorld.
	mgRelationOnCollectionWithIcons := MGRelationOnCollectionWithIcons new 
		collection: 
		{
			MGPoint point: 0@0 pointIcon: MGCrossPointIcon new.
			MGPoint point: 1@1 pointIcon: MGDotPointIcon new.
			MGPoint point: 0@1 pointIcon: MGCrossPointIcon new.
			MGPoint point: 1@0 pointIcon: MGRectanglePointIcon new
		}.
	mgPlotMorph addRelation: mgRelationOnCollectionWithIcons.
	
Instance Variables
!

!MGRelationOnCollectionWithIcons methodsFor: 'private drawing' stamp: 'mg 12/7/2013 14:26'!
drawContinuousOn: aCanvas transformation: aPlotToFormTransformation
	collection overlappingPairsDo:
	[:firstPoint :secondPoint|
		aCanvas 
			line: (aPlotToFormTransformation toFormPoint: firstPoint point) asIntegerPoint
			to: (aPlotToFormTransformation toFormPoint: secondPoint point) asIntegerPoint
			color: color
	]! !

!MGRelationOnCollectionWithIcons methodsFor: 'private drawing' stamp: 'mg 12/7/2013 15:28'!
drawNotContinuousOn: aCanvas transformation: aPlotToFormTransformation
	collection do:
	[:point||transformedPoint|
		transformedPoint := (aPlotToFormTransformation toFormPoint: point point).
		point pointIcon drawOn: aCanvas at: transformedPoint.
	]! !
